---------------------------------------------------------------
-- Universidade Federal de Pernambuco - UFPE
-- Centro de Informatica - CIn
-- Projeto: HPCIn
-- Coordenador: Manoel Eusebio
----------------------------------------------------------------
----------------------------------------------------------------
-- Modulo: mult_mat_vector.vhd
-- Descricao: Top do multiplicador matriz vetor
-- 
-- Autor(es): 	Viviane Lucy <vivianelucy@gmail.com>
--				Victor Medeiros <victorwcm@gmail.com>
--
-- Data de criacao: 21/07/08
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use IEEE.std_logic_arith.all;

-- definicao da entidade
entity mult_mat_vector is
	port(
		-- portas gerais
		Pclk				:	in std_logic;
		Prst				:	in std_logic;		
		-- portas para conexao com a placa DE2
		Pinit_button		: in std_logic;
		Prd_memory_button	: in std_logic;
		Pend_led			: out std_logic;
		Pdisplay0			: out std_logic_vector(6 downto 0);
		Pdisplay1			: out std_logic_vector(6 downto 0); 
		Pdisplay2			: out std_logic_vector(6 downto 0); 
		Pdisplay3			: out std_logic_vector(6 downto 0) 
	);
end mult_mat_vector;

-- descricao da arquitetura comportamental da entidade 
architecture arch_mult_mat_vector of mult_mat_vector is
	-- definicao dos sinais
	-- sinais de controle do multiplicador
	signal Sdone				: std_logic;
	signal Smac_vld				: std_logic;
	signal Sinit_button 		: std_logic;
	signal Srd_memory_button 	: std_logic;
	
	-- sinais relacionados a memoria que armazena a matriz
	signal Saddr_mem_matrix		: std_logic_vector(9 downto 0);
	signal Sdata_out_mem_matrix	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix		: std_logic;
	
	-- sinais relacionados a memoria que armazena o vetor
	signal Saddr_mem_vector		: std_logic_vector(6 downto 0);
	signal Sdata_out_mem_vector	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_vector	: std_logic_vector(31 downto 0);
	signal Swr_mem_vector		: std_logic;
	
	-- sinais relacionados a memoria 3 (vetor resultante)
	signal Saddr_wr_mem_out		: std_logic_vector(3 downto 0);
	signal Saddr_rd_mem_out		: std_logic_vector(3 downto 0);
	signal Swr_mem_out			: std_logic;
	signal Sdata_in_mem_out		: std_logic_vector(31 downto 0);
	signal Sdata_out_mem_out	: std_logic_vector(31 downto 0);

	-- definicao dos componentes
	-- definicao do controlador
	component mem_control
		port (
			Pclk				: 	in std_logic;
			Prst				: 	in std_logic;
		
			-- portas de conexao com as memorias
			-- porta que endereca a memoria que contem a matriz
			Paddr_mem_matrix	: out std_logic_vector(9 downto 0);
			-- porta que endereca a memoria que contem o vetor
			Paddr_mem_vector	: out std_logic_vector(6 downto 0);
			-- porta que endereca a memoria que contem o vetor de saida
			Paddr_mem_out		: out std_logic_vector(3 downto 0);
			-- porta que aciona a escrita na memoria de saida
			Pwr_mem_out			: out std_logic;
		
			-- sinais de controle
			-- porta que indica que o botao de init foi pressionado
			Pinit_button		: in std_logic;
			-- porta que indica para o MAC que ja existem dados validos
			Pmac_vld: out std_logic;
			-- sinaliza para o MAC que todos os produtos ja foram realizados
			Pdone: out std_logic
		);
	end component;
	
	-- definicao da memoria que armezena a matriz
	component memory_matrix
		port (
			address : in std_logic_vector(9 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;

	-- definicao da memoria que armazena o vetor
	component memory_vector
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;

	-- definicao da memoria dual-port que armazena o vetor de saida
	component memory_out_2p
		port (
			clock		: in std_logic;
			data		: in std_logic_vector(31 downto 0);
			rdaddress	: in std_logic_vector(3 downto 0);
			wraddress	: in std_logic_vector(3 downto 0);
			wren		: in std_logic := '1';
			q			: out std_logic_vector(31 downto 0)
		);
	end component;

	-- definicao do multiplicador acumulador de 32 bits
	component mac32bits 
		port (
			Prst 		: in std_logic;
			Pclk		: in std_logic;
			Pmult_vld 	: in std_logic;
			Pdone_ac	: in std_logic;
			Pinput_a	: in std_logic_vector (31 downto 0);
			Pinput_b	: in std_logic_vector (31 downto 0);
			Poutput		: out std_logic_vector (31 downto 0)
		);
	end component;

	-- definicao do componente responsavel por ler a memoria de saida e exibir os valores
	-- nos displays de 7 segmentos da DE2
	component memory_reader
		port (
			Prst 				: in std_logic;
			Pclk				: in std_logic;
			-- dado lido da memoria
			Pdata_in			: in std_logic_vector(31 downto 0);
			-- endereco a ser lido da memoria
			Prd_addr			: out std_logic_vector(3 downto 0);
			-- saidas para os displays de 7 segmentos
			Pdisplay0			: out std_logic_vector(6 downto 0);
			Pdisplay1			: out std_logic_vector(6 downto 0);
			Pdisplay2			: out std_logic_vector(6 downto 0);
			Pdisplay3			: out std_logic_vector(6 downto 0);
			-- entrada que dispara o inicio da leitura do conteudo da memoria
			Prd_memory_button	: in  std_logic;
			-- sinaliza o final da leitura dos dados atraves de um LED
			Pend_led			: out std_logic
		);
	end component;

	-- definicao do componente responsavel por realizar o debounce dos botoes
	component debounce
		port (
			Prst 	: in std_logic;
			Pclk	: in std_logic;
			Pin		: in std_logic;
			Pout	: out std_logic
		);
	end component;

	begin

	Sdata_in_mem_matrix 	<= x"00000000";
	Swr_mem_matrix 			<= '0';
	Sdata_in_mem_vector 	<= x"00000000";
	Swr_mem_vector 			<= '0';

	-- instancia dos componentes
	-- instancia do modulo controlador
	mem_ctrl : mem_control
        port map (
            Pclk => Pclk,
			Prst => Prst,
			Paddr_mem_matrix 	=> Saddr_mem_matrix,
			Paddr_mem_vector 	=> Saddr_mem_vector,
			Paddr_mem_out 		=> Saddr_wr_mem_out,
			Pwr_mem_out 		=> Swr_mem_out,
			Pinit_button 		=> Sinit_button,
			Pmac_vld 			=> Smac_vld,
			Pdone 				=> Sdone
		);

	-- instancia da memoria que armazena a matriz 
	mem_matrix : memory_matrix
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix,
			wren	=> swr_mem_matrix,
			q	 	=> Sdata_out_mem_matrix
		);
	
	-- instancia da memoria que armazena o vetor			
	mem_vector : memory_vector
		port map (
			address	=> Saddr_mem_vector,
			clock	=> Pclk,
			data	=> Sdata_in_mem_vector,
			wren	=> Swr_mem_vector,
			q	 	=> Sdata_out_mem_vector
		);

	-- instancia da memoria que armazena o vetor de saida
	mem_out : memory_out_2p
		port map (
			rdaddress 	=> Saddr_rd_mem_out,
			wraddress 	=> Saddr_wr_mem_out,
			clock	 	=> Pclk,
			data	 	=> Sdata_in_mem_out,
			wren	 	=> Swr_mem_out,
			q	 		=> Sdata_out_mem_out
		);
	
	mac : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mem_out
		);

	reader : memory_reader 
		port map (
			Prst 		=> Prst,
			Pclk		=> Pclk,
			Pdata_in	=> Sdata_out_mem_out,
			Prd_addr	=> Saddr_rd_mem_out,
			-- saidas para os displays de 7 segmentos
			Pdisplay0	=> Pdisplay0,
			Pdisplay1	=> Pdisplay1,
			Pdisplay2	=> Pdisplay2,
			Pdisplay3	=> Pdisplay3, 
			-- entrada que dispara o inicio da leitura do conteudo da memoria
			Prd_memory_button => Srd_memory_button,
			-- sinaliza o final da leitura atraves do LED
			Pend_led	=> Pend_led
		);

	debounce_init : debounce 
		port map (
			Prst	=> Prst,
			Pclk	=> Pclk,
			Pin		=> Pinit_button,
			Pout	=> Sinit_button
		);

	debounce_read_memory : debounce 
		port map (
			Prst	=> Prst,
			Pclk	=> Pclk,
			Pin 	=> Prd_memory_button,
			Pout	=> Srd_memory_button
		);
end arch_mult_mat_vector;