---------------------------------------------------------------
-- Universidade Federal de Pernambuco - UFPE
-- Centro de Informatica - CIn
-- Projeto: HPCIn
-- Coordenador: Manoel Eusebio
----------------------------------------------------------------
----------------------------------------------------------------
-- Modulo: mem_control.vhd
-- Descricao: 	Modulo que controla o funcionamento do 
--				multiplicador matriz vetor
-- 
-- Autor(es): 	Viviane Lucy <vivianelucy@gmail.com>
--				Victor Medeiros <victorwcm@gmail.com>
--
-- Data de criacao: 21/07/08
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;

-- definicao da entidade
entity mem_control is
	port (
		Pclk				: 	in std_logic;
		Prst				: 	in std_logic;
		
		-- portas de conexao com as memorias
		-- porta que endereca a memoria que contem a matriz
		Paddr_mem_matrix	: out std_logic_vector(6 downto 0);
		-- porta que endereca a memoria que contem o vetor
		Paddr_mem_vector	: out std_logic_vector(6 downto 0);
		
		-- sinais de controle
		-- porta que indica que o botao de init foi pressionado
		Pinit_button		: in std_logic;
		-- porta que indica para o MAC que ja existem dados validos
		Pmac_vld: out std_logic;
		-- sinaliza para o MAC que todos os produtos ja foram realizados
		Pdone: out std_logic
	);
end mem_control;

-- descricao da arquitetura da entidade
architecture arch_mem_control of mem_control is

	-- definicao dos tipos
	type ctrl_mem_state_type is (INIT, WAIT_CYCLE, READ_DATA, WAIT_3, ADDR_MEM, END_PROCESS);

	--definicao dos sinais
	signal Sctrl_mem_state	: ctrl_mem_state_type;
	signal Saddr_mem_matrix	: std_logic_vector(6 downto 0);
	signal Saddr_mem_vector	: std_logic_vector(6 downto 0);

	begin
	-- ligacao dos sinais a portas
	Paddr_mem_matrix <= Saddr_mem_matrix; 
	Paddr_mem_vector <= Saddr_mem_vector;

	-- processo que implementa o comportamento do controlador						
	process(Pclk, Prst, Pinit_button)							
		-- variaveis utilizadas no processo
		-- variavel que conta numero de operacoes do MAC
		variable Vop_counter	: integer range 0 to 100;

		begin
			-- sincronismo com o clock
			if (Pclk'event and Pclk = '1') then		
				-- se o reset foi pressionado
				if (Prst = '0') then
					Saddr_mem_matrix <= "0000000"; 
					Saddr_mem_vector <= "0000000";
					Pmac_vld <= '0';
					Vop_counter := 0;
					Sctrl_mem_state <= INIT;
					Pdone <= '0';
				else
					case Sctrl_mem_state is 
						-- estado de inicializacao
						when INIT =>
							-- caso o botao init tenha sido pressionado 
							-- inicializa as portas de saida e dispara a execucao
							if (Pinit_button = '1') then
								Saddr_mem_matrix <= "0000000"; 
								Saddr_mem_vector <= "0000000";
								Pmac_vld <= '0';
								Vop_counter := 0;
								Pdone <= '0';
								Sctrl_mem_state <= WAIT_CYCLE;
							else 
								Sctrl_mem_state <= INIT;
							end if;
							
						when WAIT_CYCLE =>
							-- espera um ciclo, tempo de resposta da memoria
							Sctrl_mem_state <= READ_DATA;
							
						when READ_DATA =>
							-- indica para o mac que os dados sao validos
							Pmac_vld <= '1';
							-- incrementa contador de operacoes do MAC
							Vop_counter := Vop_counter + 1;
							-- vai enderecar novas palavras
							Sctrl_mem_state <= WAIT_3;
						
						when WAIT_3 =>
							Pmac_vld <= '0';
							Sctrl_mem_state <= ADDR_MEM;
							
						-- estado que realiza o enderecamento de memoria
						when ADDR_MEM =>
							Pmac_vld <= '0';
							-- endereca a matriz
							Saddr_mem_matrix <= Saddr_mem_matrix + "0000001"; 
							-- endereca o vetor
							Saddr_mem_vector <= Saddr_mem_vector + "0000001";
							-- se ja realizou 100 operacoes
							if ( Vop_counter = 100) then
								-- zera contador e escreve na memoria de saida
								Vop_counter := 0;
								Pdone <= '1';
								Sctrl_mem_state <= END_PROCESS;
							else
								--retorna a uma nova leitura da memoria
								Sctrl_mem_state <= WAIT_CYCLE;
							end if;
						
						when END_PROCESS =>
								Pdone <= '0';
								-- operacao finalizada, se mantem no estado
								Sctrl_mem_state <= END_PROCESS;
					end case;
				end if;
			end if;
	end process;
end arch_Mem_Control;

